@@ -36,7 +36,6 @@
 #include <sys/wait.h>
 
 #include <ctype.h>
-#include <dlfcn.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <fnmatch.h>
@@ -66,7 +65,9 @@
 #include "rc-misc.h"
 #include "rc-plugin.h"
 
-#define SELINUX_LIB     RC_LIBDIR "/runscript_selinux.so"
+#ifdef HAVE_SELINUX
+#include "rc-selinux.h"
+#endif
 
 #define PREFIX_LOCK	RC_SVCDIR "/prefix.lock"
 
@@ -88,42 +89,6 @@ static int signal_pipe[2] = { -1, -1 };
 static RC_STRINGLIST *types_b, *types_n, *types_nu, *types_nua, *types_m;
 static RC_STRINGLIST *types_mua = NULL;
 
-#ifdef __linux__
-static void (*selinux_run_init_old)(void);
-static void (*selinux_run_init_new)(int argc, char **argv);
-
-static void
-setup_selinux(int argc, char **argv)
-{
-	void *lib_handle = NULL;
-
-	if (! exists(SELINUX_LIB))
-		return;
-
-	lib_handle = dlopen(SELINUX_LIB, RTLD_NOW | RTLD_GLOBAL);
-	if (! lib_handle) {
-		eerror("dlopen: %s", dlerror());
-		return;
-	}
-
-	selinux_run_init_old = (void (*)(void))
-	    dlfunc(lib_handle, "selinux_runscript");
-	selinux_run_init_new = (void (*)(int, char **))
-	    dlfunc(lib_handle, "selinux_runscript2");
-
-	/* Use new run_init if it exists, else fall back to old */
-	if (selinux_run_init_new)
-		selinux_run_init_new(argc, argv);
-	else if (selinux_run_init_old)
-		selinux_run_init_old();
-	else
-		/* This shouldnt happen... probably corrupt lib */
-		eerrorx("run_init is missing from runscript_selinux.so!");
-
-	dlclose(lib_handle);
-}
-#endif
-
 static void
 handle_signal(int sig)
 {
@@ -210,8 +175,7 @@ start_services(RC_STRINGLIST *list)
 			{
 				rc_service_schedule_start(service,
 				    svc->value);
-				ewarn("WARNING: %s is scheduled to started"
-				    " when %s has started",
+				ewarn("WARNING: %s will start when %s has started",
 				    svc->value, applet);
 			} else
 				service_start(svc->value);
@@ -313,15 +277,13 @@ write_prefix(const char *buffer, size_t bytes, bool *prefixed)
 	if (lock_fd != -1) {
 		while (flock(lock_fd, LOCK_EX) != 0) {
 			if (errno != EINTR) {
-				eerror("flock() failed: %s", strerror(errno));
+				ewarnv("flock() failed: %s", strerror(errno));
 				break;
 			}
 		}
 	}
-#ifdef RC_DEBUG
 	else
-		ewarn("Couldn't open the prefix lock, please make sure you have enough permissions");
-#endif
+		ewarnv("Couldn't open the prefix lock, please make sure you have enough permissions");
 
 	for (i = 0; i < bytes; i++) {
 		/* We don't prefix eend calls (cursor up) */
@@ -762,8 +724,7 @@ svc_start_deps(void)
 		}
 		rc_stringlist_free(tmplist);
 		tmplist = NULL;
-		ewarnx("WARNING: %s is scheduled to start when "
-		    "%s has started", applet, tmp);
+		ewarnx("WARNING: %s will start when %s has started", applet, tmp);
 		free(tmp);
 	}
 
@@ -1120,7 +1081,7 @@ static const char *const longopts_help[] = {
 #include "_usage.c"
 
 int
-runscript(int argc, char **argv)
+openrc_run(int argc, char **argv)
 {
 	bool doneone = false;
 	int retval, opt, depoptions = RC_DEP_TRACE;
@@ -1134,12 +1095,12 @@ runscript(int argc, char **argv)
 
 	/* Show help if insufficient args */
 	if (argc < 2 || !exists(argv[1])) {
-		fprintf(stderr, "runscript should not be run directly\n");
+		fprintf(stderr, "openrc-run should not be run directly\n");
 		exit(EXIT_FAILURE);
 	}
 
 	if (stat(argv[1], &stbuf) != 0) {
-		fprintf(stderr, "runscript `%s': %s\n",
+		fprintf(stderr, "openrc-run `%s': %s\n",
 		    argv[1], strerror(errno));
 		exit(EXIT_FAILURE);
 	}
@@ -1228,9 +1189,9 @@ runscript(int argc, char **argv)
 		eprefix(prefix);
 	}
 
-#ifdef __linux__
+#ifdef HAVE_SELINUX
 	/* Ok, we are ready to go, so setup selinux if applicable */
-	setup_selinux(argc, argv);
+	selinux_setup(argc, argv);
 #endif
 
 	deps = true;
@@ -1424,3 +1385,10 @@ runscript(int argc, char **argv)
 
 	return retval;
 }
+
+int
+runscript(int argc, char **argv)
+{
+	ewarnv("runscript is deprecated; please use openrc-run instead.");
+	return (openrc_run(argc, argv));
+}
